#!/bin/bash
########################################
#
#      This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; either version 3 of the License, or
#      (at your option) any later version.
#      
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
########################################
#
# script to rotate options and call gsecraif with a file
#
# $Date$
# $Revision$
#
########################################
function testgsecraif {
    cat $file2split | $gsecraifsplit $argumentstring -d $debuglevel
    if [ $? != 0 ]; then
        echo "Error splitting file"
        exit 1
    fi
    $gsecraifcomb -c $argumentstring -d $debuglevel > $recfilename
    if [ $? != 0 ]; then
        echo "Error combining file"
        exit 1
    fi
    diff $file2split $recfilename
    if [ $? != 0 ]; then
        echo "Fail"
        exit 1
    fi
    # remove a random component file
    fileid=$(($RANDOM%$slices))
    if [ $verbosity -gt 2 ]; then
        echo -n "  $fileid "
    fi
    if [ $fileid -lt 10 ]; then
        comp2remove="split00${fileid}"
    elif [ $fileid -lt 100 ]; then
        comp2remove="split0${fileid}"
    else
        comp2remove="split${fileid}"
    fi
    rm $comp2remove
    rm $recfilename
    $gsecraifcomb -c $argumentstring -d $debuglevel > $recfilename 2> /dev/null
    if [ $? != 0 ]; then
        echo "Error combining file"
        exit 1
    fi
    diff $file2split $recfilename
    if [ $? != 0 ]; then
        echo "Fail"
        exit 1
    else
        let passcount=${passcount}+1
        if [ $verbosity -gt 2 ]; then
            echo " passcount: ${passcount}"
        else
            echo -n "."
        fi
    
        rm $recfilename
        rm split*
    fi 
    }

function cyclearguments {
    # cycle through the arguments to gsecraif
    startslice=3	# number of slices to start with
    endslice=255 	# number of slices to end with
    slices=3	# number of slices
    rotate=0	# rotated bits
    argumentn=""	# -n argument
    argumentr=""	# -r argument
    while [ $slices -le $endslice ]; do
        rotate=0
        while [ $rotate -le 7 ]; do
            argumentstring=" -n ${slices} -r ${rotate}"
            if [ $hashes -eq 1 ]; then
                argumentstring="${argumentstring} -#"
            fi
            if [ $verbosity -gt 2 ]; then
                echo -n "slices: ${slices}"
                echo -n " "
                echo -n "rotate: ${rotate}"
            fi
            # call test function:
            testgsecraif
            let rotate=${rotate}+1
        done
        # test transpose:
        argumentstring=" -n ${slices} -t"
        if [ $hashes -eq 1 ]; then
            argumentstring="${argumentstring} -#"
        fi
        if [ $verbosity -gt 2 ]; then
            echo -n "slices: ${slices}"
            echo -n " "
            echo -n "transpose"
        fi
        # call test function:
        testgsecraif

        let slices=${slices}+1
    done
    }

function printusage {

cat << EOFTEXT

Usage $0 [options] filename

Where filename is the name of the file gsecraif will use for testing.

-v N               verbosity level (Default is 0)
-d directory-path  directory path for testing

EOFTEXT

    }

##############################################################################
# main script starts here:

gsecraifsplit="/usr/local/bin/gsecraif"
gsecraifcomb="/usr/local/bin/gsecraif"
# To test different versions of gsecraif set one of the above to one version
# and set the other function to the other version of gsecraif
# run the tests then reverse the settings above.
#
argumentstring=""
debuglevel=0
passcount=0
dirgiven=0	# flag to indicate temp test directory supplied
hashes=0
recfilename="recovered"
#verbosity=3	# level of output
verbosity=0	# level of output
if [ $(($#)) -lt 1 ]; then
  printusage
  exit 1
fi
while getopts ":v:d:h" opt; do
    case $opt in
        v)
            #echo "-v $OPTARG "
            case $OPTARG in
                0)
                    verbosity=0
                    ;;
                1)
                    verbosity=1
                    ;;
                2)
                    verbosity=2
                    ;;
                3)
                    verbosity=3
                    ;;
            esac
            ;;
        d)
            #echo "-d $OPTARG "
            temptestdir=$OPTARG
            dirgiven=1
            ;;
        h)
            hashes=1
            #echo "hashes ${hashes}"
            ;;
        \?)
            #echo "Invalid option: -$OPTARG" >&2
            printusage
            ;;
        :)
            echo "-$OPTARG requires an argument"
    esac
done
file2split=$BASH_ARGV

if [ ! -f $file2split ]; then
    echo "${file2split} can not be found or is not a reguarl file"
    exit 1
fi
if [ ! -r $file2split ]; then
    echo "${file2split} does not have read permission"
    exit 1
fi

if [ $dirgiven -eq 0 ]; then
    temptestdir="gsecraiftesting"
    mkdir $temptestdir > /dev/null 2>&1
fi
if [ ! -d ${temptestdir} ]; then
    echo "Problem with temp test directory ${temptestdir}"
    exit 1
fi
cd $temptestdir
if [ $verbosity -gt 2 ]; then
    echo "gsecraif test"
    echo "File to split: ${file2split}"
fi

# cycle through arguments and test:
cyclearguments
cd ..
# clean up
if [ $dirgiven -eq 0 ]; then
    rmdir $temptestdir
fi
echo
exit 0


